/* $Id: open.c,v 1.4 1996/02/06 01:10:34 seb Exp $ */
/* ARC BIOS open routine

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <arc/proto.h>
#include <debug.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>

int 
open (const char *pathname, int flags,...)
{
  OPEN_MODE arcmode;
  _STATUS st;
  ULONG fd;

  if (debuglevel >= 2)
    {
      printf ("open(): entered with pathname=%s, flags=%d\r\n",
	      pathname, flags);
      fflush (stdout);
    }

  switch (flags & O_ACCMODE)
    {
    case O_RDONLY:
      arcmode = OpenReadOnly;
      break;
    case O_WRONLY:
      arcmode = (flags & O_CREAT) ? CreateWriteOnly : OpenWriteOnly;
      break;
    case O_RDWR:
      arcmode = (flags & O_CREAT) ? CreateReadWrite : OpenReadWrite;
      break;
    default:
      errno = EINVAL;
      return -1;
    }

  if (debuglevel >= 2)
    {
      printf ("open(): Calling ARC Open(%s,%d,%p)\r\n",
	      pathname, arcmode, &fd);
      fflush (stdout);
    }
  st = Open (pathname, arcmode, &fd);
  if (st != ESUCCESS)
    {
      errno = st;
      return -1;
    }
  if (debuglevel >= 2)
    {
      printf ("open(): ARC Open() returned fd=%d\r\n", fd);
      fflush (stdout);
    }
  return fd;
}
