/* $Id: sleep.c,v 1.4 1996/02/06 01:10:36 seb Exp $ */
/* ARC BIOS sleep routine

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <arc/proto.h>
#include <arc/time.h>

/*
 * sleep()
 * Wait an arbitrary amount of time specified in seconds.
 * FIXME: This might fail shortly before midnight at the end of a month :-)
 */
unsigned int 
sleep (unsigned int seconds)
{
  TIMEINFO *current;
  TIMEINFO end;

  current = (TIMEINFO *) GetTime ();	/* get current time */

  /* calculate time to reach */

  end.Hundredths = current->Hundredths;
  end.Seconds = current->Seconds + (seconds % 60);
  end.Minutes = current->Minutes + ((seconds / 60) % 60);
  end.Hour = current->Hour + ((seconds / 3600) % 24);
  end.Day = current->Day;
  end.Month = current->Month;
  end.Year = current->Year;

  /* check overflow */

  if (end.Seconds > 59)
    {
      end.Seconds %= 60;
      end.Minutes++;
    }
  if (end.Minutes > 59)
    {
      end.Minutes %= 60;
      end.Hour++;
    }
  if (end.Hour > 23)
    {
      end.Hour %= 24;
      end.Day++;
    }

  /* wait until time is reached */

  while (current->Day != end.Day ||
	 current->Hour != end.Hour ||
	 current->Minutes != end.Minutes ||
	 current->Seconds != end.Seconds)
    current = (TIMEINFO *) GetTime ();

  return (0);
}

/*
 * usleep()
 * Sleep an arbitrary amount of time specified in microseconds.
 * NOTE: ARC GetTime() always returns Hundredths = 0,
 *       therefortime is quantized to one second.
 */
unsigned int 
usleep (unsigned int micros)
{
  return (sleep (micros / 100000));
}
