/* $Id: write.c,v 1.4 1996/02/06 01:10:38 seb Exp $ */
/* ARC BIOS write routine

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <asm/bootinfo.h>
#include <arc/arc.h>
#include <arc/proto.h>
#include <debug.h>
#include <stdio.h>
#include <unistd.h>

/*
 * There is definately a bug in Read() when reading more than 512 bytes.
 * Don't know whether this bug exists in Write() bug somehow that seems
 * probable to me, so use the workaround here, too.
 */
#define BLOCKSIZE  0x200	/* Mips Magnum 4000 BIOS crashes when */
				/* writing more than 512 bytes at once ! */

/*
 * Write to file in BLOCKSIZE bytes chunks. This workaround is way to slow
 * on other systems, so use it only for the broken Magnums.
 */
static __inline__ ssize_t 
splitted_write (int fd, const __ptr_t buf, size_t count)
{
  ULONG Count;
  ULONG n;
  _STATUS st;
  ULONG c;

  Count = 0;
  while (count)
    {
      n = (count < BLOCKSIZE ? count : BLOCKSIZE);
      if (debuglevel >= 3)
	{
	  /*
	   * Turn all printing off to avoid a infinite indirect recursion
	   * loop printf() -> write() -> printf () ...
	   */
	  int save = option_debuglevel;
	  option_debuglevel = 0;

	  printf ("write(): count=%d, n=%d, Count=%d\r\n", count, n, Count);
	  fflush (stdout);
	  option_debuglevel = save;
	}

      st = Write (fd, (PVOID) buf, (ULONG) n, &c);
      if (st != ESUCCESS)
	{
	  errno = st;
	  return -1;
	}
      Count += c;
      buf += c;
      count -= c;
    }

  return (ssize_t) Count;
}

static __inline__ ssize_t 
default_write (int fd, const __ptr_t buf, size_t count)
{
  ULONG Count;
  _STATUS st;

  st = Write (fd, (PVOID) buf, (ULONG) count, &Count);
  if (st != ESUCCESS)
    {
      errno = st;
      return -1;
    }

  return (ssize_t) Count;
}

ssize_t 
write (int fd, const __ptr_t buf, size_t count)
{
  ssize_t c;

  if (__splitted_rw)
    c = splitted_write (fd, buf, count);
  else
    c = default_write (fd, buf, count);

  return c;
}
