/* $Id: __sflags.c,v 1.4 1996/02/06 01:10:25 seb Exp $ */
/* Parse flags for fopen & Co.
   Copyright (C) 1995 by Ralf Baechle

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <arc/errno.h>
#include <stdio.h>
#include <fcntl.h>

/*
 * Return the flags for a given mode string.
 * Return 0 on error and set errno.
 *
 * Just ignores everything behind a '+' and b flags.
 */
int
__sflags (const char *mode, int *flagp)
{
  int flags;

  if (mode == NULL)
    {
      errno = EINVAL;
      return 0;
    }
  switch (mode[0])
    {
    case 'r':
      flags = O_RDONLY;
      break;
    case 'w':
      flags = O_WRONLY | O_CREAT | O_TRUNC;
      break;
    case 'a':
      flags = O_WRONLY | O_CREAT | O_APPEND;
      break;
    default:
      errno = EINVAL;
      return 0;
    }

  if (mode[1] == '+' ||
      (mode[1] == 'b' && mode[2] == '+'))
    {
      flags = (flags & ~O_ACCMODE) | O_RDWR;
    }

  *flagp = flags;
  return 1;
}
