/* $Id$ */
/* Identify the ARC system we're running on
   Copyright (C) 1996 by Stoned Elipot.
   This file is derived from the previous identify.c which copyright
   notice follows:
   
   Copyright (C) 1994, 1995 by Waldorf Electronics,
   written by Andreas Busse and Ralf Baechle

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <linux/linkage.h>

#include <stdio.h>
#include <unistd.h>
#include <asm/addrspace.h>
#include <asm/bootinfo.h>
#include <asm/mipsregs.h>
#include <milo.h>
#include <arc/arc.h>


/* used for communication from machine specifics identification routines */
unsigned long totalram_found = 0;

static unsigned long val_icache_size = 0;	/* FIXME: cachectl.S needs these */
static unsigned long val_icache_linesize = 0;
static unsigned long val_dcache_size = 0;
static unsigned long val_dcache_linesize = 0;
static unsigned long val_scache_size = 0;
static unsigned long val_scache_linesize = 0;

/* value holders for tags data of Mips Magnum 4000 *and* Olivetti M700-10 */
static unsigned long machtype_MIPS_MAGNUM_4000 = MACH_MIPS_MAGNUM_4000;
static unsigned long machtype_MIPS_OLIVETTI_M700 = MACH_OLIVETTI_M700;
static unsigned long vram_base_MIPS_MAGNUM_4000 = 0xe0800000;

/* value holders for tags data of Deskstation rPC44 and Tyne */
static unsigned long machtype_DESKSTATION_RPC44 = MACH_DESKSTATION_RPC44;
static unsigned long machtype_DESKSTATION_TYNE = MACH_DESKSTATION_TYNE;
static unsigned long vram_base_DESKSTATION_TYNE = 0xe10a0000;

/* value holders for tags data of Acer Pica 61 */
static unsigned long machtype_ACER_PICA_61 = MACH_ACER_PICA_61;
static unsigned long vram_base_ACER_PICA_61 = 0xe08b8000;	/* for 80x25, 640x480 */

/* values for tags of SNI RM200 PCI */
unsigned long machtype_SNI_RM200 = MACH_SNI_RM200_PCI;
unsigned long vram_base_SNI_RM200_PCI = 0xb00b8000;	/* VGA text mode */

/* value holders for tags data of an unknown machine */
static unsigned long machtype_UNKNOWN = MACH_UNKNOWN;
static unsigned long machgroup_UNKNOWN = MACH_GROUP_UNKNOWN;

/* value holders for tags data of JAZZ machines */
static unsigned long machgroup_JAZZ = MACH_GROUP_JAZZ;

/* value holders for tags data of generic ARC machines */
static unsigned long machgroup_ARC = MACH_GROUP_ARC;

/* values for tags of ARC SNI machines */
unsigned long machgroup_SNI_RM = MACH_GROUP_SNI_RM;

/* value holders for misc tags data */
static unsigned long tlb_entries_48 = 48;
static struct drive_info_struct drive_info_DUMMY = DEFAULT_DRIVE_INFO;
static unsigned long unsigned_long_low_loadaddr = KSEGX (KERNELBASE);
static unsigned long unsigned_long_KSEG0_8Mo = KSEG0ADDR (0x800000);
static unsigned long unsigned_long_KSEG0_32Mb = KSEG0ADDR (0x2000000);
static unsigned long unsigned_long_512x1024 = 512 * 1024;

/* value holders of various size for zeroed tags data */
static unsigned char unsigned_char_ZERO = 0;
static unsigned int unsigned_int_ZERO = 0;
static unsigned long unsigned_long_ZERO = 0;

/* these add clearity to the arrays below */
/* Hum,... or not ?? ;) -stoned. */
#undef TAG
#undef TAGVAL
#define TAG(t,l)  {tag_##t,(l)}
#define TAGVAL(v) (void*)&(v)

/* default tag list for Mips Magnum 4000 */
static tag_def taglist_mips_magnum_4000[] =
{
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_MIPS_MAGNUM_4000)},
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_JAZZ)},
  {TAG (scache_size, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},
  /*  {TAG(dma_cache_size, ULONGSIZE), TAGVAL(unsigned_long_ZERO)}, */
  /*  {TAG(dma_cache_base, ULONGSIZE), TAGVAL(unsigned_long_ZERO)}, */
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (vram_base_MIPS_MAGNUM_4000)},
  {TAG (dummy, 0), NULL}	/* must be the last one */
};

/* default tag list for Olivetti M700-10 */
static tag_def taglist_olivetti_m700[] =
{
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_MIPS_OLIVETTI_M700)},
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_JAZZ)},
  {TAG (scache_size, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},
  /*  {TAG(dma_cache_size, ULONGSIZE), TAGVAL(unsigned_long_ZERO)}, */
  /*  {TAG(dma_cache_base, ULONGSIZE), TAGVAL(unsigned_long_ZERO)}, */
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (vram_base_MIPS_MAGNUM_4000)},
  {TAG (dummy, 0), NULL}	/* must be the last one */
};


/* default tag list for Deskstation rPC44 */
  /*
   * RB: This entry is absolutly untested - I don't have
   * even a hint about how the L2 cache works, so I make
   * the guess that it works just like the Tyne board.
   */
static tag_def taglist_deskstation_rpc44[] =
{
  /* WARNING ! memupper tag as to be the first in this list 
   * as long as we don't find a way to discover ourselves 
   * how much RAM there is in such a box 
   */
  {TAG (memupper, ULONGSIZE), TAGVAL (unsigned_long_KSEG0_8Mo)},
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_DESKSTATION_RPC44)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_ARC)},
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  {TAG (scache_size, ULONGSIZE), TAGVAL (unsigned_long_512x1024)},
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},
  {TAG (dma_cache_size, ULONGSIZE), TAGVAL (unsigned_long_512x1024)},
  {TAG (dma_cache_base, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (vram_base_DESKSTATION_TYNE)},
  {TAG (dummy, 0), NULL}	/* must be the last one */
};

/* default tag list for Deskstation Tyne */
static tag_def taglist_deskstation_tyne[] =
{
  /* WARNING ! see the comment in the definition of 
   * taglist_deskstation_rpc44[] above about memupper tag
   */
  {TAG (memupper, ULONGSIZE), TAGVAL (unsigned_long_KSEG0_8Mo)},
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_DESKSTATION_TYNE)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_ARC)},
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  {TAG (scache_size, ULONGSIZE), TAGVAL (unsigned_long_512x1024)},
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},
  {TAG (dma_cache_size, ULONGSIZE), TAGVAL (unsigned_long_512x1024)},
  {TAG (dma_cache_base, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (vram_base_DESKSTATION_TYNE)},
  {TAG (dummy, 0), NULL}	/* must be the last one */
};

/* default tag list for Acer Pica 61 */
static tag_def taglist_acer_pica_61[] =
{
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_ACER_PICA_61)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_JAZZ)},
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  {TAG (scache_size, ULONGSIZE), TAGVAL (unsigned_long_512x1024)},
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},
  /*  {TAG(dma_cache_size, ULONGSIZE), TAGVAL(unsigned_long_ZERO)}, */
  /*  {TAG(dma_cache_base, ULONGSIZE), TAGVAL(unsigned_long_ZERO)}, */
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (vram_base_ACER_PICA_61)},
  {TAG (dummy, 0), NULL}	/* must be the last one */
};

/* tag list for SNI RM200 PCI */
tag_def taglist_sni_rm200_pci[] =
{
  /*
   * For now I wire this to 32mb - see rm200-pci.c.
   */
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_SNI_RM200)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_SNI_RM)},
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (vram_base_SNI_RM200_PCI)},
  {TAG (dummy, 0), NULL}	/* must be the last one */
};

/* default tag list for unknown machine 
 * You don't really hope that Linux boot on such a system, don't you?
 *
 * Nb: commented tags are tags for which value is found out
 *     by identify() or set at runtime
 */
static tag_def taglist_unknown[] =
{
  /* WARNING ! memupper tag has to be the first tag here */
  {TAG (memupper, ULONGSIZE), TAGVAL (unsigned_long_KSEG0_8Mo)},
  {TAG (machtype, ULONGSIZE), TAGVAL (machtype_UNKNOWN)},
  {TAG (machgroup, ULONGSIZE), TAGVAL (machgroup_UNKNOWN)},
  /*  {TAG(cputype, ULONGSIZE), TAGVAL()}, *//* detected */
  {TAG (memlower, ULONGSIZE), TAGVAL (unsigned_long_low_loadaddr)},
  /*  {TAG(icache_size, ULONGZISE), TAGVAL()}, *//* detected */
  /*  {TAG(icache_linesize, ULONGZISE), TAGVAL()}, *//* detected */
  /*  {TAG(dcache_size, ULONGZISE), TAGVAL()}, *//* detected */
  /*  {TAG(dcache_linesize, ULONGZISE), TAGVAL()}, *//* detected */
  {TAG (scache_size, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},	/* should be detected */
  /*  {TAG(scache_linesize, ULONGSIZE), TAGVAL()}, *//* how this one is set/detected ?? */
  {TAG (tlb_entries, ULONGSIZE), TAGVAL (tlb_entries_48)},	/* true for all R4x00 except R4300i (32) */
  {TAG (dma_cache_size, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  {TAG (dma_cache_base, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  /* {TAG(ramdisk_size, ULONGSIZE), TAGVAL()}, *//* runtime set */
  /* {TAG(ramdisk_base, ULONGSIZE), TAGVAL()}, *//* runtime set */
  /*  {TAG(mount_root_rdonly, ULONGSIZE), TAGVAL()}, *//* runtime set */
  {TAG (drive_info, DRVINFOSIZE), TAGVAL (drive_info_DUMMY)},
  {TAG (vram_base, ULONGSIZE), TAGVAL (unsigned_long_ZERO)},
  /*  {TAG(command_line, CMDLINESIZE), TAGVAL()}, *//* runtime set */
  {TAG (dummy, 0), NULL}	/* must be the last one */
};


/*
 *  Structure containing infos to get Milo identify
 *  a machine.
 */
struct machine_struct
  {
    char *SystemId;		/* same as value returned by getSystemId()  */

    char *VendorId;		/* VendorId is the same as the value        */
    /* returned by getVendorId() or (char*)NULL */

    char *identify_string;	/* string printed by identify() about the machine */

    machine_func *machine_ident;	/* specific system identification routine */
    /* (see for Acer Pica 61) */

    machine_func *machine_init;	/* specific system initialization before kernel */
    /* launch (see for DeskStation Tyne) */
    tag_def *taglist;		/* array of default tag to create */
  };


/*
 * Array containing all the machine_struct corresponding to
 * known machines
 *
 * BEWARE!!! Order is revelant. See below in identify() on 
 *           how to add entries 
 */
static struct machine_struct machinetbl[] =
{
  {"DESKTECH-ARCStation I", (char *) NULL, "DeskStation rPC44",
   deskstation_rpc44_bi, (machine_func *) NULL,
   taglist_deskstation_rpc44
  },

  {"DESKTECH-TYNE", (char *) NULL, "DeskStation Tyne",
   deskstation_tyne_bi, deskstation_tyne_init,
   taglist_deskstation_tyne
  },

  {"PICA-61", (char *) NULL, "ACER PICA-61",
   acer_pica_61_bi, (machine_func *) NULL,
   taglist_acer_pica_61
  },

  /* This one is nearly a Magnum: it's spelled O L I V E T T I */
  /* but pronounced Magnum :) */
  {"Microsoft-Jazz", "OLI00000", "Olivetti M700-10 (Jazz)",
   magnum_type_bi, (machine_func *) NULL,
   taglist_olivetti_m700
  },

  /* Someone please give me the vendor id string for MIPS */
  /* Thanks -stoned */
  {"Microsoft-Jazz", (char *) NULL, "Mips Magnum 4000 (Jazz)",
   magnum_type_bi, (machine_func *) NULL,
   taglist_mips_magnum_4000
  },

  /*
   * SNI RM200 PCI
   */
  {"RM200PCI", "RM200", "SNI RM200 PCI",
   rm200_pci_bi, (machine_func *) NULL,
   taglist_sni_rm200_pci
  },

  /*
   * must be the last element.
   */
{(char *) NULL, (char *) NULL, " Not identified -- using default parameters",
 (machine_func *) NULL, (machine_func *) NULL,
 taglist_unknown
  }
};

/*
 * identify()
 */
void 
identify (void)
{
  char *VendorId;		/* ARC vendor ID */
  char *ProductId;		/* ARC product ID */
  char *SystemId;		/* ARC system ID */
  int prid;
  unsigned long type;
  int config;
  int i;
  int machine_idx = 0;		/* index in machinetbl[] corresponding */
  /* to the one identified */
  struct DisplayInfo *di;	/* for gathering ARC video infos */

  /*
   * Get system information from ARC BIOS 
   */
  VendorId = getVendorId ();
  ProductId = getProductId ();
  SystemId = getSystemId ();

  /* How machinetbl[] is being scanned? We search for a matching SystemId
   * and VendorId fields or for a matching SystemId and a null VendorId
   * So in machinetbl[], elements like SystemId = "XXX" VendorId="YYY"
   * *should* come before elements like SystemId = "XXX" VendorId = NULL
   * This way specific distinctions are possible and a generic entry for each
   * SystemId can be used. See for Olivetti M700 and Magnum 4000.  No match
   * at all permits use of the default entry.
   */
  for (i = 0; i < (sizeof (machinetbl) / sizeof (machinetbl[0])); i++)
    {
      /* BE CAREFULL: the order of this tests is important!
       * A NULL pointer shouldn't be passed to strncmp() or be prepared for
       * TLBE. So tests on NULL pointers are done before calls to strncmp().
       */
      if ((machinetbl[i].SystemId == (char *) NULL)
	  || ((!strncmp (machinetbl[i].SystemId, SystemId, 8))
	      && ((machinetbl[i].VendorId == (char *) NULL)
		  || (!strncmp (machinetbl[i].VendorId, VendorId, 8)))))
	{
	  machine_idx = i;
	  break;
	}
    }

  /* special machine identification stuff ? 
   * if any, call it early, we may need it for finding out the 
   * memory upper limit 
   */
  if (machinetbl[machine_idx].machine_ident != (machine_func *) NULL)
    machinetbl[machine_idx].machine_ident ();

  /* create the default tag entries for the box we just had identified */
  bi_TagAddList (machinetbl[machine_idx].taglist);

  /*
   * Get CPU Info
   */
  prid = read_32bit_cp0_register (CP0_PRID);
  type = decodecputype (prid);
  bi_TagAdd (tag_cputype, ULONGSIZE, (void *) &type);

  /*
   * Get cache and cache line sizes
   */
  config = read_32bit_cp0_register (CP0_CONFIG);
  val_icache_size = 1 << (12 + ((config & 0xe00) >> 9));
  val_dcache_size = 1 << (12 + ((config & 0x1c0) >> 6));
  val_icache_linesize = 16 << ((config & 0x20) >> 5);
  val_dcache_linesize = 16 << ((config & 0x10) >> 4);

  bi_TagAdd (tag_icache_size, ULONGSIZE, (void *) &val_icache_size);
  bi_TagAdd (tag_icache_linesize, ULONGSIZE, (void *) &val_icache_linesize);
  bi_TagAdd (tag_dcache_size, ULONGSIZE, (void *) &val_dcache_size);
  bi_TagAdd (tag_dcache_linesize, ULONGSIZE, (void *) &val_dcache_linesize);

  val_scache_size = getSecondaryCacheSize ();
  /* For now the secondary cache size is set from the sample taglists */
  /* bi_TagAdd(tag_scache_size, ULONGSIZE, (void*)&val_scache_size); */

  /* set the function pointer for specific machine initialisation */
  mach_init = machinetbl[machine_idx].machine_init;

  /* get the ARC Ddisplay information */
  if ((di = getDisplayInfo ()))
    {
      bi_TagAdd (tag_arcdisplayinfo, sizeof (struct DisplayInfo), (void *) di);
    }
  else
    {
      /* Milo used to do that, so I let it there -Stoned. */
      printf ("Display Initialization failed.\r\n");
      sleep (10);
      exit (EXIT_FAILURE);
    }

  if (option_verbose)
    {
      printf ("ARC Vendor  ID          : [");
      for (i = 0; i < 8; i++)	/* CHANGER CA */
	printf ("%c", (unsigned char) VendorId[i]);
      printf ("]\r\n");
      printf ("ARC Product ID          : [");
      for (i = 0; i < 8; i++)	/* CHANGER CA */
	printf ("%02x", (unsigned char) ProductId[i]);
      printf ("]\r\n");
      printf ("ARC System  ID          : [%s]\r\n", SystemId);
      printf ("ARC CPU ID              : [%s]\r\n", getCpuType ());
      printf ("Processor type          : [%s I%02x V%d.%d]\r\n",
	      cpuname (type),
	      prid >> 8, (prid & 0xf0) >> 4, prid & 0x0f);
      printf ("Primary ICache size     : [0x%04x bytes]\r\n",
	      (unsigned int) val_icache_size);
      printf ("Primary ICache line size: [0x%04x bytes]\r\n",
	      (unsigned int) val_icache_linesize);
      printf ("Primary DCache size     : [0x%04x bytes]\r\n",
	      (unsigned int) val_dcache_size);
      printf ("Primary DCache line size: [0x%04x bytes]\r\n",
	      (unsigned int) val_dcache_linesize);
      printf ("Secondary cache size    : ");
      if (val_scache_size != SIZE_UNKNOWN)
	printf ("[0x%4lx]\r\n", val_scache_size);
      else
	printf ("[unknown]\r\n");
      if (totalram_found)
	printf ("Total RAM detected      : %ldmb\r\n", totalram_found >> 20);
      printf ("ARC system identified as: %s\r\n\r\n", machinetbl[machine_idx].identify_string);
    }
  return;
}
