/* $Id: bitags.c,v 1.5 1996/02/06 01:10:02 seb Exp $ */
/* handles the tags passed to the kernel
   Copyright (C) 1995,96 by Stoned Elipot <Stoned.Elipot@fnet.fr>
   written by Stoned Elipot from an original idea of
   Ralf Baechle <ralf@waldorf-gmbh.de>

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <string.h>
#include <linux/string.h>
#include <asm/bootinfo.h>
#include <milo.h>

/* Tags creation little How-To:
   Tags space grows like a stack from memory_upper. First we put at address 
   next_tag minus sizeof(struct tag) a 'tag structure' (type tag
   defined in bootinfo.h) containing info on the tag, then below it 
   it we put tag's data.   */

/* Next_tag contains the address of the lower bound of the tags space 
   it's initialised from various means - see below - at the memory upper 
   limit (real or a fake one if we were unable to determine the box's memory
   size) so we lost a... byte at top of memory, what a shame.  :)  */
static unsigned long next_tag = (unsigned long) NULL;

/* Create a tag.  The parameters are the tag's, its size, and a pointer
   on a buffer holding its associated data (can be NULL if no data belongs
   to the tag).  It returns 0 when successfull and 1 when it fails.  */
int
bi_TagAdd (enum bi_tag type, unsigned long size, void *data)
{
  tag t;
  unsigned long addr;

  t.tag = type;
  t.size = size;
  /* if next_tag equals NULL it means it's the first tag we're asked to create */
  if (next_tag == (unsigned long) NULL)
    {
      /* if memory_upper not equals NULL it means that identify() was able  
         to figure out how much memory is present in the box so we initialize 
         next_tag from it.  */
      if (memory_upper != (unsigned long) NULL)
	{
	  next_tag = memory_upper;
	}
      else
	/* Else we rely on the fact that the first tag we create for a box
	   for which we don't know how RAM it gots is a tag of type
	   tag_memupper.  This is ensured by the first entry in the
	   defaults tag list for such a box (see identify.c).  First
	   we check this.  */
	{
	  if (type != tag_memupper)
	    {
	      /* This usually mean there is a consistency problem in
	         identify.c */
	      return 1;
	    }
	  else
	    {
	      /* Ok it's a memupper tag: we put it's value in 
	         memory_upper so launch() can pass it to the kernel 
	         in register a0 and we initialize next_tag.  */
	      next_tag = *(unsigned long *) data;
/*              memcpy((void*)&memory_upper, data, size); */
	      memory_upper = *((unsigned long *) data);
	    }
	}
    }
  /* We put the tag structure.  */
  addr = next_tag - (sizeof (tag));
  if (debuglevel >= 2)
    {
      printf ("bi_TagAdd: adding tag struct at %08lx, tag: %d, size: %08lx\r\n",
	      addr, t.tag, t.size);
    }
  memcpy ((void *) addr, (void *) &t, (size_t) (sizeof (tag)));
  /* We put the tag's data if any.  */
  if (size != 0)
    {
      addr = addr - size;
      if (debuglevel >= 2)
	{
	  printf ("bi_TagAdd: adding tag value at %08lx\r\n", addr);
	}
      memcpy ((void *) addr, data, (size_t) (t.size));
    }
  /* set next_tag ready for the next tag creation.  */
  next_tag = addr;
  return 0;
}

/* Create tags from a "null-terminated" array of tag 
   (tag.type  in the last tag_def struct in the array 
   must be 'tag_dummy').  It returns 0 when successfull and 1 
   when it fails.  */
int
bi_TagAddList (tag_def * taglist)
{
  int ret_val = 0;
  for (; (taglist->t.tag != tag_dummy) && (!ret_val); taglist++)
    {
      /* We assume this tag is present in the default taglist
         for the box we're running on.  */
      if (taglist->t.tag == tag_memlower)
	{
	  memory_lower = (unsigned long) (*((unsigned long *) taglist->d));
	}
      ret_val = bi_TagAdd (taglist->t.tag, taglist->t.size, taglist->d);
    }
  return ret_val;
}

/* Parse the tags present in upper memory to find out a pecular one
   We pass it the type of the tag to find. It returns a NULL pointer
   when it fails or a pointer on the tag structure found.  */
/* Note: it's just a 'prototype', the kernel's one is in 
   arch/mips/kernel/setup.c  */
/* tag* */
/* bi_TagFind(enum bi_tag type) */
/* { */
/*   tag* t; */
/*   t = (tag*)(mach_mem_upper - sizeof(tag)); */
/*   while((t->tag != tag_dummy) && (t->tag != type)) */
/*       t = (tag*)(NEXTTAGPTR(t)); */
/*   if (t->tag == tag_dummy)   */
/*     { */
/*       return (tag*)NULL; */
/*     } */
/*   return t; */
/* } */
