/* $Id$ */
/* MIPS cpu identification routines
   Copyright (C) 1994, 1995 by Waldorf Electronics,
   written by Andreas Busse and Ralf Baechle

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <asm/bootinfo.h>

/*
 * decodecputype()
 * Extract cpu type from Processor Id
 * NOTE: This is not complete. Fill in here anything you know about.
 *
 * ID is organized as follows:
 *   Bits 31:16   always 0
 *   Bits 15:08   CPU Implementation
 *   Bits 07:04   CPU Version
 *   Bits 03:00   CPU Revision
 *
 * FIXME: CPU dependent stuff should go into a global array.
 */
unsigned long 
decodecputype (int id)
{
  switch (id >> 8)
    {
    case 0x01:
      return CPU_R2000;

    case 0x02:
      return CPU_R3000;

    case 0x03:
      return CPU_R6000;

    case 0x04:			/* any of R4000xx and R4400xx */
      switch (id & 0xf0)
	{
	case 0x00:		/* R4000 */
	case 0x10:
	case 0x20:
	case 0x30:
	  return CPU_R4000PC;

	case 0x40:		/* R4400 V4.x with daddiu bug */
	case 0x50:		/* R4400 V5.x */
	  return CPU_R4400PC;

	default:
	  return CPU_UNKNOWN;
	}
      break;

    case 0x05:			/* LSI Logic R3000 derivative */
      return CPU_R3000;

    case 0x06:
      return CPU_R6000A;

    case 0x07:
      return CPU_R3051;		/* IDT 3051,3052 etc. */

    case 0x09:
      return CPU_R10000;

    case 0x0a:
      return CPU_R4200;

    case 0x10:
      return CPU_R8000;

    case 0x20:
      return CPU_R4600;		/* R4600 V1.x has an icache bug! */

    case 0x21:
/*
 * Warning: The information for type 0x21 is contradictory.
 */
#if 1
      return CPU_R4700;		/* IDT R4700 */
#else
      return CPU_R3000;		/* Sony R3000 derivative */
#endif

    case 0x22:
      return CPU_R3000;		/* Toshiba R3000 derivate */

    case 0x23:
      return CPU_R3000;		/* NKK R3000 derivative */

    default:
      return CPU_UNKNOWN;

    }
}

static char *cpu_name_strings[] = CPU_NAMES;
/*
 * cpuname()
 * Translate cputype into a processor name
 * FIXME: This should also go into a global array of cpu informations.
 *        Some of these CPUs are too stupid (no MMU) to run Linux.
 */
char *
cpuname (int type)
{
  if (type > CPU_LAST)
    type = CPU_UNKNOWN;

  return cpu_name_strings[type];
}
