/* $Id: expandpath.c,v 1.4 1996/02/06 01:10:30 seb Exp $ */
/* ARC BIOS path expansion routine
   Copyright (C) by Stoned Elipot <Stoned.Elipot@fnet.fr>

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <linux/ctype.h>
#include <string.h>
#include <stdlib.h>

/*
 * expand_path()
 * try to make the Pandoriens' life easier when
 * dealing with pathnames.
 */
char *
expand_path (char *org, char *default_path)
{
  char *idx, *temp, *subst;
  int len;
  char *retval = org;

  /* if there is a '(' assume it's a complete ARC path - see arc(5) - */
  /* so give up.                                                      */
  if (!(strchr (org, '(')))
    {
      /* but if there is ':' and no '(' it should be something like       */
      /* 'A:\foo.bar' let's check this out...                             */
      if ((idx = strchr (org, ':')))
	{
	  for (temp = org, len = 0;
	       ((isalpha (*temp) && temp != idx));
	       temp++, len++)
	    ;
	  if (temp == idx)	/* ok, there is only letters before the ':' */
	    {
	      /* use temp again, it's previous value is not usefull anymore  */
	      temp = (char *) malloc (len + 2);
	      strncpy (temp, org, len + 1);	/* get the 'A' plus the ':' */
	      *(temp + len + 1) = '\0';
	      subst = getenv (temp);	/* get it's value from environment */
	      free (temp);	/* poor temp, we nearly don't need it :) */
	      if (subst)	/* ok, 'A:' is out there, so let's do the job */
		{
		  idx++;	/* idx now points after the ':' */
		  /* compute the len of the final path */
		  len = strlen (subst) + strlen (idx) + 1;
		  retval = (char *) malloc (len);
		  strcpy (retval, subst);	/* assemble the substitued path */
		  strcat (retval, idx);		/* all done */
		}		/* not in the environment, let open() deal with this badly */
	      /* formed path */
	    }			/* not only letters before the ':', same punition :) */
	}
      else
	/* so there is nothing special in this path ? Hum, let's throw in the */
	/* default path if not NULL, hoping that the result is ok :) */
	{
	  if (default_path)
	    {
	      len = strlen (default_path) + strlen (org) + 1;
	      retval = (char *) malloc (len);
	      strcpy (retval, default_path);
	      strcat (retval, org);
	    }
	}
    }
  return retval;
}
