/* $Id: magnum.c,v 1.5 1996/03/17 16:50:56 seb Exp $ */
/* Magnum type specific routines: used for MIPS Magnum 4000 
   and Olivetti M700-10
   Copyright (C) 1996 Stoned Elipot & Luc Beurton

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <linux/tty.h>
#include <asm/io.h>
#include <asm/bootinfo.h>
#include <asm/jazz.h>
#include <g364.h>
#include <stdio.h>
#include <milo.h>

#define MB (1024*1024)

/*
 * magnum_type_bi()
 * 
 * Detects the installed RAM size
 * See Olivetti's 'M700-10 RISC Personnal Computer - 
 * hardware-architecture and function' pp 3-4, 3-5
 *
 * Detects some video (G364 adapter) related parameters
 */
int 
magnum_type_bi (void)
{
  unsigned int gcregv;
  unsigned char bankindice;
  unsigned int banksize = 0;
  unsigned int totalram = 0;
  int i;
  int bankenabled;
  int tweenbankenabled;
  struct screen_info si = DEFAULT_SCREEN_INFO;
  unsigned int mx, my;

  si.orig_x = 0;		/* start at col. 0, line 0 */
  si.orig_y = 0;
  G364_get_display (&mx, &my);
  si.orig_video_cols = mx / 8;
  si.orig_video_lines = my / 16;
  bi_TagAdd (tag_screen_info, sizeof (struct screen_info), (void *) &si);

  gcregv = (unsigned int) (*((unsigned int *) JAZZ_R4030_CONFIG));
  for (i = 0; i < 2; i++)
    {
      bankenabled = ((int) ((gcregv >> (4 + i)) & 0x1));
      if (bankenabled)
	{
	  tweenbankenabled = ((int) ((gcregv >> (4 + i + 2)) & 0x1));
	  bankindice = ((unsigned char) ((gcregv >> ((i % 2) ? 2 : 0)) & 0x3));
	  switch (bankindice)
	    {
	    case 0:
	      banksize = (tweenbankenabled ? 2 : 1);
	      break;
	    case 1:
	      banksize = (tweenbankenabled ? 8 : 4);
	      break;
	    case 2:
	      banksize = (tweenbankenabled ? 32 : 16);
	      break;
	    case 3:
	      banksize = 0;
	      break;
	    }
	  totalram += (banksize << 2) * MB;
	}
    }
  memory_upper = KSEGX (KERNELBASE) + totalram;
  /* we don't need to create a memupper tag, but for consistency */
  bi_TagAdd (tag_memupper, ULONGSIZE, (void *) &memory_upper);
  return 0;
}
