/* $Id$ */
/* Pandora debugger/monitor main program
   Copyright (C) 1995 by Andreas Busse

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <string.h>
#include <milo.h>
#include <pandora.h>

/* from parser.c */
int parsit (char *line, char ***array);

/*
 * Local Prototypes
 */
int nop (void);
int help (void);
void prompt (void);

/*
 * Command table
 */
static struct cmdent
  {
    char *cmd_name;
    int (*cmd_fctn) ();
  }
cmdtbl[] =
{
  {CMD_BOOT, pandora_boot},
  {CMD_DUMP, dump},
  {CMD_DUMPTLB, dumptlb},
  {CMD_FILL, fill},
  {CMD_MEMORY, memory},
  {CMD_PROBE, probe},
  {CMD_STRINGS, strings},
  {CMD_HELP, help},
  {CMD_CP0REGS, cp0regs},
  {CMD_QUIT, quit},
  {CMD_DISASM, disasm},
  {CMD_NOP, nop},				/* must be the last command */
};

#define NUMCMDS ((sizeof(cmdtbl)/sizeof(cmdtbl[0]))-1)

/*
 * pandora()
 * Debugger/Monitor main loop
 */
int 
pandora (void)
{
  char line[MAXBUFLEN];
  char **argp = NULL;
  char *cp;
  int argnum, i;

  /*
   * Now get commands one at a time and execute the
   * appropriate routine to deal with them.
   */

  for (;;)
    {
      prompt ();

#ifdef STANDALONE
      if (fgets (line, sizeof (line), stdin) == NULL)
#else
      if (gets (line) == NULL)
#endif
	break;

      /*
       * Strip trailing CR-LF
       */
      cp = line + strlen (line) - 1;
      while (cp >= line && (*cp == '\n' || *cp == '\r'))
	*cp-- = '\0';

      /*
       * Null command
       */
      if ((argnum = parsit (line, &argp)) == 0)
	continue;

      for (i = 0; i < NUMCMDS; ++i)
	if (!strcasecmp (cmdtbl[i].cmd_name, argp[0]))
	  break;

      if (i < NUMCMDS)
	{
	  (*cmdtbl[i].cmd_fctn) (argnum, argp);
	}
      else
	{
	  printf ("Unrecognized command.\r\n");
	  fflush (stdout);
	}
    }

  return 0;
}

int
quit (int argc, char *argv[])
{
  exit (0);
}

/*
 * nop()
 * NOP Function, only for debugging purposes.
 */
int 
nop (void)
{
  return 0;
}

/*
 * help()
 * Print info about available commands
 */
int 
help (void)
{
  printf ("\r\nKnown commands:\r\n\r\n");
  printf ("boot [kernel_file [ramdisk_file]]\r\n     [-- kernel_arguments]\t\tcontinue boot process\r\n");
  printf ("dump [-b|h|w] start [end]\t\tdump memory contents\r\n");
  printf ("dumptlb [all] [filename]\t\tdump TLB contents\r\n");
  printf ("fill [-b|h|w] start [end] val\t\tfill memory with value\r\n");
  printf ("memory\t\t\t\t\tshow available memory chunks\r\n");
  printf ("probe vaddr\t\t\t\tprobe TLB for a virtual address\r\n");
  printf ("strings start [end]\t\t\tsearch memory for printable strings\r\n");
  printf ("cp0regs\t\t\t\t\tdump the CP0 registers\r\n");
  printf ("disasm [-r] [-p] start end [filename]\tdisassemble from memory\r\n");
  printf ("quit\t\t\t\t\tquit pandora\r\n");
  printf ("\r\n\r\n");

  return 0;
}

/*
 * prompt()
 * Prompt for a new command
 */
void 
prompt (void)
{
  printf ("Pandora> ");
  fflush (stdout);
}
