/* $Id$ */
/* Linux/MIPS kernel ARC BIOS debugger/monitor for ARC compliant boxes.
   Copyright (C) 1996 by Stoned Elipot.
 
   based on old milo.c, here is the copyright notice
   from this previous source file
 
   Copyright (C) 1994, 1995 by Waldorf Electronics,
   written by Ralf Baechle and Andreas Busse
 
   Loosly based on bootstrap.c for Linux/68k,
   Copyright (C) 1993 by Hamish Macdonald, Greg Harp

This file is part of Milo.

Milo is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2, or (at your option)
any later version.

Milo is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with Milo; see the file COPYING.  If not, write to
the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA 02111-1307, USA.  */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <milo.h>
#include <pandora.h>
#include <mkargs.h>

/* mallocinit() from libarc.a need that */
u_long loadaddr = LOADADDR;

/* options */
int option_verbose = 1;
int option_interactive = 0;

/* default milo path for various operation */
char default_path[256] = "\0";

/* for now we just use the aout loader */
extern struct loader loader_aout;
struct loader *kld = &loader_aout;

/* set Milo/Pandora behaviour in various common functions */
int behaviour = BEHAVE_PANDORA;

/* local prototype */
void usage (void);

/*
 * main()
 */
int 
main (int argc, char *argv[])
{
  int ch;
  int realargc = argc;
  char **realargv = argv;
  char args[1024];
  char *newargv[256];
  char *OSLoadOptions;

  printf (version_string ,"BIOS debugger/monitor");
  printf("\r\n");
  printf ("Copyright (C) Waldorf Electronics and others 1994, 1995, 1996\r\n\r\n");

  /* WARNING: temporary use of newargv[0] */
  newargv[0] = getenv ("MiloDefaultPath");	/* get the default path from environment */
  if (!newargv[0])
    strcpy (default_path, DEFAULT_PATH);	/* DEFAULT_PATH *has* a trailing '\' */
  else
    {
      strncpy (default_path, newargv[0], 256);
      /* MiloDefaultPath should not have a trailing '/' or '\'. */
      /* This seems to be the standard for ARC BIOS env var */
      strcat (default_path, "\\");
    }

  /* analyse arguments */
  OSLoadOptions = getenv ("OSLoadOptions");
  if (argc <= 1)
    {
      /*
       * If we don't get arguments, we'll default to the environment
       * variables values. Happens only when run manual.
       */
      strncpy (args, argv[0], 1023);
      if (OSLoadOptions)
	{
	  strncat (args, " ", 1023);
	  strncat (args, OSLoadOptions, 1023);
	}
      args[strlen (args)] = '\0';
      mkargs (&realargc, argv, args);
    }
  else
    {
      int i;
      /*
       * Grep the OSLoadOptions from the option string
       * and use it as the real option string
       */
      for (i = 1; i < argc; i++)
	{
	  if (strncasecmp ("OSLoadOptions=", argv[i], 13) == 0)
	    {
	      strncpy (args, argv[0], 1023);
	      strncat (args, " ", 1023);
	      strncat (args, argv[i] + 14, 1023);
	      *(args + strlen (args)) = '\0';
	      mkargs (&realargc, newargv, args);
	      realargv = newargv;
	    }
	}
    }

  if (realargc > 1)
    {
      while ((ch = getopt (realargc, realargv, "d")) != EOF)
	switch (ch)
	  {
	  case 'd':
	    option_debuglevel++;
	    break;
	  case '?':
	  default:
	    usage ();
	  }
    }

  /* Identify the system we're running on */
  identify ();

  /* go to pandora shell */
  pandora ();

  exit (0);
}

/*
 * usage()
 *
 * Options:
 * -d - enable debugging (default is no debugging)
 * -v - verbose output
 */
void 
usage (void)
{
  fprintf (stderr, "Usage:\r\n"
	   "\tpandora [-d] [-v]\r\n");
  exit (EXIT_FAILURE);
}
